=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2006-05-02 21:17:38 +0000
+++ bzrlib/tests/test_urlutils.py	2006-05-02 22:43:51 +0000
@@ -27,6 +27,74 @@
 
 class TestUrlToPath(TestCase):
     
+    def test_basename(self):
+        # bzrlib.urlutils.basename
+        # Test bzrlib.urlutils.split()
+        basename = urlutils.basename
+        if sys.platform == 'win32':
+            self.assertRaises(InvalidURL, basename, 'file:///path/to/foo')
+            self.assertEqual('foo', basename('file:///C|/foo'))
+            self.assertEqual('', basename('file:///C|/'))
+        else:
+            self.assertEqual('foo', basename('file:///foo'))
+            self.assertEqual('', basename('file:///'))
+
+        self.assertEqual('foo', basename('http://host/path/to/foo'))
+        self.assertEqual('foo', basename('http://host/path/to/foo/'))
+        self.assertEqual('',
+            basename('http://host/path/to/foo/', exclude_trailing_slash=False))
+        self.assertEqual('path', basename('http://host/path'))
+        self.assertEqual('', basename('http://host/'))
+        self.assertEqual('', basename('http://host'))
+        self.assertEqual('path', basename('http:///nohost/path'))
+
+        self.assertEqual('path', basename('random+scheme://user:pass@ahost:port/path'))
+        self.assertEqual('path', basename('random+scheme://user:pass@ahost:port/path/'))
+        self.assertEqual('', basename('random+scheme://user:pass@ahost:port/'))
+
+        # relative paths
+        self.assertEqual('foo', basename('path/to/foo'))
+        self.assertEqual('foo', basename('path/to/foo/'))
+        self.assertEqual('', basename('path/to/foo/',
+            exclude_trailing_slash=False))
+        self.assertEqual('foo', basename('path/../foo'))
+        self.assertEqual('foo', basename('../path/foo'))
+
+    def test_dirname(self):
+        # Test bzrlib.urlutils.dirname()
+        dirname = urlutils.dirname
+        if sys.platform == 'win32':
+            self.assertRaises(InvalidURL, dirname, 'file:///path/to/foo')
+            self.assertEqual('file:///C|/', dirname('file:///C|/foo'))
+            self.assertEqual('file:///C|/', dirname('file:///C|/'))
+        else:
+            self.assertEqual('file:///', dirname('file:///foo'))
+            self.assertEqual('file:///', dirname('file:///'))
+
+        self.assertEqual('http://host/path/to', dirname('http://host/path/to/foo'))
+        self.assertEqual('http://host/path/to', dirname('http://host/path/to/foo/'))
+        self.assertEqual('http://host/path/to/foo',
+            dirname('http://host/path/to/foo/', exclude_trailing_slash=False))
+        self.assertEqual('http://host/', dirname('http://host/path'))
+        self.assertEqual('http://host/', dirname('http://host/'))
+        self.assertEqual('http://host', dirname('http://host'))
+        self.assertEqual('http:///nohost', dirname('http:///nohost/path'))
+
+        self.assertEqual('random+scheme://user:pass@ahost:port/',
+            dirname('random+scheme://user:pass@ahost:port/path'))
+        self.assertEqual('random+scheme://user:pass@ahost:port/',
+            dirname('random+scheme://user:pass@ahost:port/path/'))
+        self.assertEqual('random+scheme://user:pass@ahost:port/',
+            dirname('random+scheme://user:pass@ahost:port/'))
+
+        # relative paths
+        self.assertEqual('path/to', dirname('path/to/foo'))
+        self.assertEqual('path/to', dirname('path/to/foo/'))
+        self.assertEqual('path/to/foo',
+            dirname('path/to/foo/', exclude_trailing_slash=False))
+        self.assertEqual('path/..', dirname('path/../foo'))
+        self.assertEqual('../path', dirname('../path/foo'))
+
     def test_function_type(self):
         if sys.platform == 'win32':
             self.assertEqual(urlutils._win32_local_path_to_url, urlutils.local_path_to_url)
@@ -57,22 +125,57 @@
         to_url = urlutils._win32_local_path_to_url
         self.assertEqual('file:///C|/path/to/foo',
             to_url('C:/path/to/foo'))
-        self.assertEqual('file:///d|/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s',
+        self.assertEqual('file:///D|/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s',
             to_url(u'd:/path/to/r\xe4ksm\xf6rg\xe5s'))
 
     def test_win32_local_path_from_url(self):
         from_url = urlutils._win32_local_path_from_url
         self.assertEqual('C:/path/to/foo',
             from_url('file:///C|/path/to/foo'))
-        self.assertEqual(u'd:/path/to/r\xe4ksm\xf6rg\xe5s',
+        self.assertEqual(u'D:/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file:///d|/path/to/r%C3%A4ksm%C3%B6rg%C3%A5s'))
-        self.assertEqual(u'd:/path/to/r\xe4ksm\xf6rg\xe5s',
+        self.assertEqual(u'D:/path/to/r\xe4ksm\xf6rg\xe5s',
             from_url('file:///d|/path/to/r%c3%a4ksm%c3%b6rg%c3%a5s'))
 
         self.assertRaises(InvalidURL, from_url, '/path/to/foo')
         # Not a valid _win32 url, no drive letter
         self.assertRaises(InvalidURL, from_url, 'file:///path/to/foo')
 
+    def test_split(self):
+        # Test bzrlib.urlutils.split()
+        split = urlutils.split
+        if sys.platform == 'win32':
+            self.assertRaises(InvalidURL, split, 'file:///path/to/foo')
+            self.assertEqual(('file:///C|/', 'foo'), split('file:///C|/foo'))
+            self.assertEqual(('file:///C|/', ''), split('file:///C|/'))
+        else:
+            self.assertEqual(('file:///', 'foo'), split('file:///foo'))
+            self.assertEqual(('file:///', ''), split('file:///'))
+
+        self.assertEqual(('http://host/path/to', 'foo'), split('http://host/path/to/foo'))
+        self.assertEqual(('http://host/path/to', 'foo'), split('http://host/path/to/foo/'))
+        self.assertEqual(('http://host/path/to/foo', ''),
+            split('http://host/path/to/foo/', exclude_trailing_slash=False))
+        self.assertEqual(('http://host/', 'path'), split('http://host/path'))
+        self.assertEqual(('http://host/', ''), split('http://host/'))
+        self.assertEqual(('http://host', ''), split('http://host'))
+        self.assertEqual(('http:///nohost', 'path'), split('http:///nohost/path'))
+
+        self.assertEqual(('random+scheme://user:pass@ahost:port/', 'path'),
+            split('random+scheme://user:pass@ahost:port/path'))
+        self.assertEqual(('random+scheme://user:pass@ahost:port/', 'path'),
+            split('random+scheme://user:pass@ahost:port/path/'))
+        self.assertEqual(('random+scheme://user:pass@ahost:port/', ''),
+            split('random+scheme://user:pass@ahost:port/'))
+
+        # relative paths
+        self.assertEqual(('path/to', 'foo'), split('path/to/foo'))
+        self.assertEqual(('path/to', 'foo'), split('path/to/foo/'))
+        self.assertEqual(('path/to/foo', ''),
+            split('path/to/foo/', exclude_trailing_slash=False))
+        self.assertEqual(('path/..', 'foo'), split('path/../foo'))
+        self.assertEqual(('../path', 'foo'), split('../path/foo'))
+
     def test_strip_trailing_slash(self):
         sts = urlutils.strip_trailing_slash
         if sys.platform == 'win32':

=== modified file 'bzrlib/urlutils.py'
--- bzrlib/urlutils.py	2006-05-02 21:17:38 +0000
+++ bzrlib/urlutils.py	2006-05-02 22:43:51 +0000
@@ -18,6 +18,8 @@
 
 """A collection of function for handling URL operations."""
 
+import os
+from posixpath import split as _posix_split
 import urllib
 import sys
 
@@ -25,6 +27,33 @@
 import bzrlib.osutils
 
 
+def basename(url, exclude_trailing_slash=True):
+    """Return the last component of a URL.
+
+    :param url: The URL in question
+    :param exclude_trailing_slash: If the url looks like "path/to/foo/"
+        ignore the final slash and return 'foo' rather than ''
+    :return: Just the final component of the URL. This can return ''
+        if you don't exclude_trailing_slash, or if you are at the
+        root of the URL.
+    """
+    return split(url, exclude_trailing_slash=exclude_trailing_slash)[1]
+
+
+def dirname(url, exclude_trailing_slash=True):
+    """Return the parent directory of the given path.
+
+    :param url: Relative or absolute URL
+    :param exclude_trailing_slash: Remove a final slash
+        (treat http://host/foo/ as http://host/foo, but
+        http://host/ stays http://host/)
+    :return: Everything in the URL except the last path chunk
+    """
+    # TODO: jam 20060502 This was named dirname to be consistent
+    #       with the os functions, but maybe "parent" would be better
+    return split(url, exclude_trailing_slash=exclude_trailing_slash)[0]
+
+
 def escape(relpath):
     """Escape relpath to be a valid url."""
     if isinstance(relpath, unicode):
@@ -48,6 +77,24 @@
     return escape(bzrlib.osutils.relpath(base, path))
 
 
+def _find_scheme_and_separator(url):
+    """Find the scheme separator (://) and the first path separator
+
+    This is just a helper functions for other path utilities.
+    It could probably be replaced by urlparse
+    """
+    scheme_loc = url.find('://')
+    if scheme_loc == -1:
+        return None, None
+
+    # Find the path separating slash
+    # (first slash after the ://)
+    first_path_slash = url.find('/', scheme_loc+3)
+    if first_path_slash == -1:
+        return scheme_loc, None
+    return scheme_loc, first_path_slash
+
+
 # jam 20060502 Sorted to 'l' because the final target is 'local_path_from_url'
 def _posix_local_path_from_url(url):
     """Convert a url like file:///path/to/foo into /path/to/foo"""
@@ -107,15 +154,43 @@
     MIN_ABS_FILEURL_LENGTH = len('file:///C|/')
 
 
-def basename(url, exclude_trailing_slash=True):
-    """Return the last component of a URL.
+def split(url, exclude_trailing_slash=True):
+    """Split a URL into its parent directory and a child directory.
 
-    :param url: The URL in question
-    :param exclude_trailing_slash: If the url looks like "path/to/foo/"
-        ignore the final slash and return 'foo' rather than ''
+    :param url: A relative or absolute URL
+    :param exclude_trailing_slash: Strip off a final '/' if it is part
+        of the path (but not if it is part of the protocol specification)
     """
-    if exclude_trailing_slash:
-        url = strip_trailing_slash(url)
+    scheme_loc, first_path_slash = _find_scheme_and_separator(url)
+
+    if first_path_slash is None:
+        # We have either a relative path, or no separating slash
+        if scheme_loc is None:
+            # Relative path
+            if exclude_trailing_slash and url.endswith('/'):
+                url = url[:-1]
+            return _posix_split(url)
+        else:
+            # Scheme with no path
+            return url, ''
+
+    # We have a fully defined path
+    url_base = url[:first_path_slash] # http://host, file://
+    path = url[first_path_slash:] # /file/foo
+
+    if sys.platform == 'win32' and url.startswith('file:///'):
+        # Strip off the drive letter
+        if path[2:3] not in '\\/':
+            raise InvalidURL(url, 
+                'win32 file:/// paths need a drive letter')
+        url_base += path[1:4] # file:///C|/
+        path = path[3:]
+
+    if exclude_trailing_slash and len(path) > 1 and path.endswith('/'):
+        path = path[:-1]
+    head, tail = _posix_split(path)
+    return url_base + head, tail
+
 
 def strip_trailing_slash(url):
     """Strip trailing slash, except for root paths.
@@ -146,16 +221,13 @@
         # of a win32 path is actually the drive letter
         if len(url) > MIN_ABS_FILEURL_LENGTH:
             return url[:-1]
-    scheme_loc = url.find('://')
-    if scheme_loc == -1:
+    scheme_loc, first_path_slash = _find_scheme_and_separator(url)
+    if scheme_loc is None:
         # This is a relative path, as it has no scheme
         # so just chop off the last character
         return url[:-1]
 
-    # Find the path separating slash
-    # (first slash after the ://)
-    first_path_slash = url.find('/', scheme_loc+3)
-    if first_path_slash == -1 or first_path_slash == len(url)-1:
+    if first_path_slash is None or first_path_slash == len(url)-1:
         # Don't chop off anything if the only slash is the path
         # separating slash
         return url

